@@ -182,7 +182,7 @@ static void rugged_repo_new_with_backend(git_repository **repo, VALUE rb_path, V
 
 	int error = 0;
 
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 	path = StringValueCStr(rb_path);
 
 	if (rb_obj_is_kind_of(rb_backend, rb_cRuggedBackend) == Qfalse) {
@@ -291,7 +291,7 @@ static VALUE rb_git_repo_open_bare(int argc, VALUE *argv, VALUE klass)
 	}
 
 	if (!repo) {
-		Check_Type(rb_path, T_STRING);
+		FilePathValue(rb_path);
 
 		error = git_repository_open_bare(&repo, StringValueCStr(rb_path));
 		rugged_exception_check(error);
@@ -335,7 +335,7 @@ static VALUE rb_git_repo_new(int argc, VALUE *argv, VALUE klass)
 	VALUE rb_path, rb_options;
 
 	rb_scan_args(argc, argv, "10:", &rb_path, &rb_options);
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 
 	error = git_repository_open(&repo, StringValueCStr(rb_path));
 	rugged_exception_check(error);
@@ -377,7 +377,7 @@ static VALUE rb_git_repo_init_at(int argc, VALUE *argv, VALUE klass)
 	int error;
 
 	rb_scan_args(argc, argv, "11:", &rb_path, &rb_is_bare, &rb_options);
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 
 	if (!NIL_P(rb_options)) {
 		/* Check for `:backend` */
@@ -474,7 +474,7 @@ static VALUE rb_git_repo_clone_at(int argc, VALUE *argv, VALUE klass)
 
 	rb_scan_args(argc, argv, "21", &url, &local_path, &rb_options_hash);
 	Check_Type(url, T_STRING);
-	Check_Type(local_path, T_STRING);
+	FilePathValue(local_path);
 
 	parse_clone_options(&options, rb_options_hash, &remote_payload);
 
@@ -1180,7 +1180,7 @@ static VALUE rb_git_repo_hashfile(VALUE self, VALUE rb_path, VALUE rb_type)
 	int error;
 	git_oid oid;
 
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 
 	error = git_odb_hashfile(&oid,
 		StringValueCStr(rb_path),
@@ -1455,7 +1455,7 @@ static VALUE rb_git_repo_discover(int argc, VALUE *argv, VALUE klass)
 		across_fs = rugged_parse_bool(rb_across_fs);
 	}
 
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 
 	error = git_repository_discover(
 		&repository_path,
@@ -1467,7 +1467,7 @@ static VALUE rb_git_repo_discover(int argc, VALUE *argv, VALUE klass)
 	rugged_exception_check(error);
 
 	error = git_repository_open(&repo, repository_path.ptr);
-	git_buf_free(&repository_path);
+	git_buf_dispose(&repository_path);
 
 	rugged_exception_check(error);
 
@@ -1502,15 +1502,6 @@ static VALUE flags_to_rb(unsigned int flags)
 	return rb_flags;
 }
 
-static int rugged__status_cb(const char *path, unsigned int flags, void *payload)
-{
-	rb_funcall((VALUE)payload, rb_intern("call"), 2,
-		rb_str_new_utf8(path), flags_to_rb(flags)
-	);
-
-	return GIT_OK;
-}
-
 static VALUE rb_git_repo_file_status(VALUE self, VALUE rb_path)
 {
 	unsigned int flags;
@@ -1518,7 +1509,7 @@ static VALUE rb_git_repo_file_status(VALUE self, VALUE rb_path)
 	git_repository *repo;
 
 	Data_Get_Struct(self, git_repository, repo);
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 	error = git_status_file(&flags, repo, StringValueCStr(rb_path));
 	rugged_exception_check(error);
 
@@ -1527,8 +1518,10 @@ static VALUE rb_git_repo_file_status(VALUE self, VALUE rb_path)
 
 static VALUE rb_git_repo_file_each_status(VALUE self)
 {
-	int error;
+	int error, exception;
+	size_t i, nentries;
 	git_repository *repo;
+	git_status_list *list;
 
 	Data_Get_Struct(self, git_repository, repo);
 
@@ -1537,13 +1530,30 @@ static VALUE rb_git_repo_file_each_status(VALUE self)
 			"A block was expected for iterating through "
 			"the repository contents.");
 
-	error = git_status_foreach(
-		repo,
-		&rugged__status_cb,
-		(void *)rb_block_proc()
-	);
-
+	error = git_status_list_new(&list, repo, NULL);
 	rugged_exception_check(error);
+
+	nentries = git_status_list_entrycount(list);
+	for (i = 0; i < nentries; i++) {
+		const git_status_entry *entry;
+		const char *path;
+		VALUE args;
+
+		entry = git_status_byindex(list, i);
+
+		path = entry->head_to_index ?
+		       entry->head_to_index->old_file.path :
+		       entry->index_to_workdir->old_file.path;
+		args = rb_ary_new3(2, rb_str_new_utf8(path), flags_to_rb(entry->status));
+		rb_protect(rb_yield, args, &exception);
+		if (exception != 0)
+			break;
+	}
+	git_status_list_free(list);
+
+	if (exception != 0)
+		rb_jump_tag(exception);
+
 	return Qnil;
 }
 
@@ -2365,7 +2375,7 @@ static VALUE rb_git_repo_attributes(int argc, VALUE *argv, VALUE self)
 	rb_scan_args(argc, argv, "12", &rb_path, &rb_names, &rb_options);
 
 	Data_Get_Struct(self, git_repository, repo);
-	Check_Type(rb_path, T_STRING);
+	FilePathValue(rb_path);
 
 	if (!NIL_P(rb_options)) {
 		Check_Type(rb_options, T_FIXNUM);
